from setuptools import setup

setup(
    name='yandex-quasar-cli',
    version='0.1.2',
    packages=['yandex_quasar_cli', 'yandex_quasar_cli.quasar_types'],
    url='https://github.com/p1ne/yandex-quasar-cli',
    license='Apache-2.0',
    author='p1ne',
    author_email='',
    description='Command line interface for Yandex Smart Home API'
)
